# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)
from chromewhip.protocol import debugger as Debugger
from chromewhip.protocol import dom as DOM
from chromewhip.protocol import network as Network
from chromewhip.protocol import runtime as Runtime
from chromewhip.protocol import runtime as Runtime
from chromewhip.protocol import emulation as Emulation

# FrameId: Unique frame identifier.
FrameId = str

# Frame: Information about the Frame on the page.
class Frame(ChromeTypeBase):
    def __init__(self,
                 id: Union['str'],
                 loaderId: Union['Network.LoaderId'],
                 url: Union['str'],
                 securityOrigin: Union['str'],
                 mimeType: Union['str'],
                 parentId: Optional['str'] = None,
                 name: Optional['str'] = None,
                 unreachableUrl: Optional['str'] = None,
                 ):

        self.id = id
        self.parentId = parentId
        self.loaderId = loaderId
        self.name = name
        self.url = url
        self.securityOrigin = securityOrigin
        self.mimeType = mimeType
        self.unreachableUrl = unreachableUrl


# FrameResource: Information about the Resource on the page.
class FrameResource(ChromeTypeBase):
    def __init__(self,
                 url: Union['str'],
                 type: Union['Network.ResourceType'],
                 mimeType: Union['str'],
                 lastModified: Optional['Network.TimeSinceEpoch'] = None,
                 contentSize: Optional['float'] = None,
                 failed: Optional['bool'] = None,
                 canceled: Optional['bool'] = None,
                 ):

        self.url = url
        self.type = type
        self.mimeType = mimeType
        self.lastModified = lastModified
        self.contentSize = contentSize
        self.failed = failed
        self.canceled = canceled


# FrameResourceTree: Information about the Frame hierarchy along with their cached resources.
class FrameResourceTree(ChromeTypeBase):
    def __init__(self,
                 frame: Union['Frame'],
                 resources: Union['[FrameResource]'],
                 childFrames: Optional['[FrameResourceTree]'] = None,
                 ):

        self.frame = frame
        self.childFrames = childFrames
        self.resources = resources


# FrameTree: Information about the Frame hierarchy.
class FrameTree(ChromeTypeBase):
    def __init__(self,
                 frame: Union['Frame'],
                 childFrames: Optional['[FrameTree]'] = None,
                 ):

        self.frame = frame
        self.childFrames = childFrames


# ScriptIdentifier: Unique script identifier.
ScriptIdentifier = str

# TransitionType: Transition type.
TransitionType = str

# NavigationEntry: Navigation history entry.
class NavigationEntry(ChromeTypeBase):
    def __init__(self,
                 id: Union['int'],
                 url: Union['str'],
                 userTypedURL: Union['str'],
                 title: Union['str'],
                 transitionType: Union['TransitionType'],
                 ):

        self.id = id
        self.url = url
        self.userTypedURL = userTypedURL
        self.title = title
        self.transitionType = transitionType


# ScreencastFrameMetadata: Screencast frame metadata.
class ScreencastFrameMetadata(ChromeTypeBase):
    def __init__(self,
                 offsetTop: Union['float'],
                 pageScaleFactor: Union['float'],
                 deviceWidth: Union['float'],
                 deviceHeight: Union['float'],
                 scrollOffsetX: Union['float'],
                 scrollOffsetY: Union['float'],
                 timestamp: Optional['Network.TimeSinceEpoch'] = None,
                 ):

        self.offsetTop = offsetTop
        self.pageScaleFactor = pageScaleFactor
        self.deviceWidth = deviceWidth
        self.deviceHeight = deviceHeight
        self.scrollOffsetX = scrollOffsetX
        self.scrollOffsetY = scrollOffsetY
        self.timestamp = timestamp


# DialogType: Javascript dialog type.
DialogType = str

# AppManifestError: Error while paring app manifest.
class AppManifestError(ChromeTypeBase):
    def __init__(self,
                 message: Union['str'],
                 critical: Union['int'],
                 line: Union['int'],
                 column: Union['int'],
                 ):

        self.message = message
        self.critical = critical
        self.line = line
        self.column = column


# LayoutViewport: Layout viewport position and dimensions.
class LayoutViewport(ChromeTypeBase):
    def __init__(self,
                 pageX: Union['int'],
                 pageY: Union['int'],
                 clientWidth: Union['int'],
                 clientHeight: Union['int'],
                 ):

        self.pageX = pageX
        self.pageY = pageY
        self.clientWidth = clientWidth
        self.clientHeight = clientHeight


# VisualViewport: Visual viewport position, dimensions, and scale.
class VisualViewport(ChromeTypeBase):
    def __init__(self,
                 offsetX: Union['float'],
                 offsetY: Union['float'],
                 pageX: Union['float'],
                 pageY: Union['float'],
                 clientWidth: Union['float'],
                 clientHeight: Union['float'],
                 scale: Union['float'],
                 zoom: Optional['float'] = None,
                 ):

        self.offsetX = offsetX
        self.offsetY = offsetY
        self.pageX = pageX
        self.pageY = pageY
        self.clientWidth = clientWidth
        self.clientHeight = clientHeight
        self.scale = scale
        self.zoom = zoom


# Viewport: Viewport for capturing screenshot.
class Viewport(ChromeTypeBase):
    def __init__(self,
                 x: Union['float'],
                 y: Union['float'],
                 width: Union['float'],
                 height: Union['float'],
                 scale: Union['float'],
                 ):

        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.scale = scale


# FontFamilies: Generic font families collection.
class FontFamilies(ChromeTypeBase):
    def __init__(self,
                 standard: Optional['str'] = None,
                 fixed: Optional['str'] = None,
                 serif: Optional['str'] = None,
                 sansSerif: Optional['str'] = None,
                 cursive: Optional['str'] = None,
                 fantasy: Optional['str'] = None,
                 pictograph: Optional['str'] = None,
                 ):

        self.standard = standard
        self.fixed = fixed
        self.serif = serif
        self.sansSerif = sansSerif
        self.cursive = cursive
        self.fantasy = fantasy
        self.pictograph = pictograph


# FontSizes: Default font sizes.
class FontSizes(ChromeTypeBase):
    def __init__(self,
                 standard: Optional['int'] = None,
                 fixed: Optional['int'] = None,
                 ):

        self.standard = standard
        self.fixed = fixed


# ClientNavigationReason: 
ClientNavigationReason = str

class Page(PayloadMixin):
    """ Actions and events related to the inspected page belong to the page domain.
    """
    @classmethod
    def addScriptToEvaluateOnLoad(cls,
                                  scriptSource: Union['str'],
                                  ):
        """Deprecated, please use addScriptToEvaluateOnNewDocument instead.
        :param scriptSource: 
        :type scriptSource: str
        """
        return (
            cls.build_send_payload("addScriptToEvaluateOnLoad", {
                "scriptSource": scriptSource,
            }),
            cls.convert_payload({
                "identifier": {
                    "class": ScriptIdentifier,
                    "optional": False
                },
            })
        )

    @classmethod
    def addScriptToEvaluateOnNewDocument(cls,
                                         source: Union['str'],
                                         worldName: Optional['str'] = None,
                                         ):
        """Evaluates given script in every frame upon creation (before loading frame's scripts).
        :param source: 
        :type source: str
        :param worldName: If specified, creates an isolated world with the given name and evaluates given script in it.
This world name will be used as the ExecutionContextDescription::name when the corresponding
event is emitted.
        :type worldName: str
        """
        return (
            cls.build_send_payload("addScriptToEvaluateOnNewDocument", {
                "source": source,
                "worldName": worldName,
            }),
            cls.convert_payload({
                "identifier": {
                    "class": ScriptIdentifier,
                    "optional": False
                },
            })
        )

    @classmethod
    def bringToFront(cls):
        """Brings page to front (activates tab).
        """
        return (
            cls.build_send_payload("bringToFront", {
            }),
            None
        )

    @classmethod
    def captureScreenshot(cls,
                          format: Optional['str'] = None,
                          quality: Optional['int'] = None,
                          clip: Optional['Viewport'] = None,
                          fromSurface: Optional['bool'] = None,
                          ):
        """Capture page screenshot.
        :param format: Image compression format (defaults to png).
        :type format: str
        :param quality: Compression quality from range [0..100] (jpeg only).
        :type quality: int
        :param clip: Capture the screenshot of a given region only.
        :type clip: Viewport
        :param fromSurface: Capture the screenshot from the surface, rather than the view. Defaults to true.
        :type fromSurface: bool
        """
        return (
            cls.build_send_payload("captureScreenshot", {
                "format": format,
                "quality": quality,
                "clip": clip,
                "fromSurface": fromSurface,
            }),
            cls.convert_payload({
                "data": {
                    "class": str,
                    "optional": False
                },
            })
        )

    @classmethod
    def captureSnapshot(cls,
                        format: Optional['str'] = None,
                        ):
        """Returns a snapshot of the page as a string. For MHTML format, the serialization includes
iframes, shadow DOM, external resources, and element-inline styles.
        :param format: Format (defaults to mhtml).
        :type format: str
        """
        return (
            cls.build_send_payload("captureSnapshot", {
                "format": format,
            }),
            cls.convert_payload({
                "data": {
                    "class": str,
                    "optional": False
                },
            })
        )

    @classmethod
    def clearDeviceMetricsOverride(cls):
        """Clears the overriden device metrics.
        """
        return (
            cls.build_send_payload("clearDeviceMetricsOverride", {
            }),
            None
        )

    @classmethod
    def clearDeviceOrientationOverride(cls):
        """Clears the overridden Device Orientation.
        """
        return (
            cls.build_send_payload("clearDeviceOrientationOverride", {
            }),
            None
        )

    @classmethod
    def clearGeolocationOverride(cls):
        """Clears the overriden Geolocation Position and Error.
        """
        return (
            cls.build_send_payload("clearGeolocationOverride", {
            }),
            None
        )

    @classmethod
    def createIsolatedWorld(cls,
                            frameId: Union['FrameId'],
                            worldName: Optional['str'] = None,
                            grantUniveralAccess: Optional['bool'] = None,
                            ):
        """Creates an isolated world for the given frame.
        :param frameId: Id of the frame in which the isolated world should be created.
        :type frameId: FrameId
        :param worldName: An optional name which is reported in the Execution Context.
        :type worldName: str
        :param grantUniveralAccess: Whether or not universal access should be granted to the isolated world. This is a powerful
option, use with caution.
        :type grantUniveralAccess: bool
        """
        return (
            cls.build_send_payload("createIsolatedWorld", {
                "frameId": frameId,
                "worldName": worldName,
                "grantUniveralAccess": grantUniveralAccess,
            }),
            cls.convert_payload({
                "executionContextId": {
                    "class": Runtime.ExecutionContextId,
                    "optional": False
                },
            })
        )

    @classmethod
    def deleteCookie(cls,
                     cookieName: Union['str'],
                     url: Union['str'],
                     ):
        """Deletes browser cookie with given name, domain and path.
        :param cookieName: Name of the cookie to remove.
        :type cookieName: str
        :param url: URL to match cooke domain and path.
        :type url: str
        """
        return (
            cls.build_send_payload("deleteCookie", {
                "cookieName": cookieName,
                "url": url,
            }),
            None
        )

    @classmethod
    def disable(cls):
        """Disables page domain notifications.
        """
        return (
            cls.build_send_payload("disable", {
            }),
            None
        )

    @classmethod
    def enable(cls):
        """Enables page domain notifications.
        """
        return (
            cls.build_send_payload("enable", {
            }),
            None
        )

    @classmethod
    def getAppManifest(cls):
        """
        """
        return (
            cls.build_send_payload("getAppManifest", {
            }),
            cls.convert_payload({
                "url": {
                    "class": str,
                    "optional": False
                },
                "errors": {
                    "class": [AppManifestError],
                    "optional": False
                },
                "data": {
                    "class": str,
                    "optional": True
                },
            })
        )

    @classmethod
    def getCookies(cls):
        """Returns all browser cookies. Depending on the backend support, will return detailed cookie
information in the `cookies` field.
        """
        return (
            cls.build_send_payload("getCookies", {
            }),
            cls.convert_payload({
                "cookies": {
                    "class": [Network.Cookie],
                    "optional": False
                },
            })
        )

    @classmethod
    def getFrameTree(cls):
        """Returns present frame tree structure.
        """
        return (
            cls.build_send_payload("getFrameTree", {
            }),
            cls.convert_payload({
                "frameTree": {
                    "class": FrameTree,
                    "optional": False
                },
            })
        )

    @classmethod
    def getLayoutMetrics(cls):
        """Returns metrics relating to the layouting of the page, such as viewport bounds/scale.
        """
        return (
            cls.build_send_payload("getLayoutMetrics", {
            }),
            cls.convert_payload({
                "layoutViewport": {
                    "class": LayoutViewport,
                    "optional": False
                },
                "visualViewport": {
                    "class": VisualViewport,
                    "optional": False
                },
                "contentSize": {
                    "class": DOM.Rect,
                    "optional": False
                },
            })
        )

    @classmethod
    def getNavigationHistory(cls):
        """Returns navigation history for the current page.
        """
        return (
            cls.build_send_payload("getNavigationHistory", {
            }),
            cls.convert_payload({
                "currentIndex": {
                    "class": int,
                    "optional": False
                },
                "entries": {
                    "class": [NavigationEntry],
                    "optional": False
                },
            })
        )

    @classmethod
    def resetNavigationHistory(cls):
        """Resets navigation history for the current page.
        """
        return (
            cls.build_send_payload("resetNavigationHistory", {
            }),
            None
        )

    @classmethod
    def getResourceContent(cls,
                           frameId: Union['FrameId'],
                           url: Union['str'],
                           ):
        """Returns content of the given resource.
        :param frameId: Frame id to get resource for.
        :type frameId: FrameId
        :param url: URL of the resource to get content for.
        :type url: str
        """
        return (
            cls.build_send_payload("getResourceContent", {
                "frameId": frameId,
                "url": url,
            }),
            cls.convert_payload({
                "content": {
                    "class": str,
                    "optional": False
                },
                "base64Encoded": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def getResourceTree(cls):
        """Returns present frame / resource tree structure.
        """
        return (
            cls.build_send_payload("getResourceTree", {
            }),
            cls.convert_payload({
                "frameTree": {
                    "class": FrameResourceTree,
                    "optional": False
                },
            })
        )

    @classmethod
    def handleJavaScriptDialog(cls,
                               accept: Union['bool'],
                               promptText: Optional['str'] = None,
                               ):
        """Accepts or dismisses a JavaScript initiated dialog (alert, confirm, prompt, or onbeforeunload).
        :param accept: Whether to accept or dismiss the dialog.
        :type accept: bool
        :param promptText: The text to enter into the dialog prompt before accepting. Used only if this is a prompt
dialog.
        :type promptText: str
        """
        return (
            cls.build_send_payload("handleJavaScriptDialog", {
                "accept": accept,
                "promptText": promptText,
            }),
            None
        )

    @classmethod
    def navigate(cls,
                 url: Union['str'],
                 referrer: Optional['str'] = None,
                 transitionType: Optional['TransitionType'] = None,
                 frameId: Optional['FrameId'] = None,
                 ):
        """Navigates current page to the given URL.
        :param url: URL to navigate the page to.
        :type url: str
        :param referrer: Referrer URL.
        :type referrer: str
        :param transitionType: Intended transition type.
        :type transitionType: TransitionType
        :param frameId: Frame id to navigate, if not specified navigates the top frame.
        :type frameId: FrameId
        """
        return (
            cls.build_send_payload("navigate", {
                "url": url,
                "referrer": referrer,
                "transitionType": transitionType,
                "frameId": frameId,
            }),
            cls.convert_payload({
                "frameId": {
                    "class": FrameId,
                    "optional": False
                },
                "loaderId": {
                    "class": Network.LoaderId,
                    "optional": True
                },
                "errorText": {
                    "class": str,
                    "optional": True
                },
            })
        )

    @classmethod
    def navigateToHistoryEntry(cls,
                               entryId: Union['int'],
                               ):
        """Navigates current page to the given history entry.
        :param entryId: Unique id of the entry to navigate to.
        :type entryId: int
        """
        return (
            cls.build_send_payload("navigateToHistoryEntry", {
                "entryId": entryId,
            }),
            None
        )

    @classmethod
    def printToPDF(cls,
                   landscape: Optional['bool'] = None,
                   displayHeaderFooter: Optional['bool'] = None,
                   printBackground: Optional['bool'] = None,
                   scale: Optional['float'] = None,
                   paperWidth: Optional['float'] = None,
                   paperHeight: Optional['float'] = None,
                   marginTop: Optional['float'] = None,
                   marginBottom: Optional['float'] = None,
                   marginLeft: Optional['float'] = None,
                   marginRight: Optional['float'] = None,
                   pageRanges: Optional['str'] = None,
                   ignoreInvalidPageRanges: Optional['bool'] = None,
                   headerTemplate: Optional['str'] = None,
                   footerTemplate: Optional['str'] = None,
                   preferCSSPageSize: Optional['bool'] = None,
                   ):
        """Print page as PDF.
        :param landscape: Paper orientation. Defaults to false.
        :type landscape: bool
        :param displayHeaderFooter: Display header and footer. Defaults to false.
        :type displayHeaderFooter: bool
        :param printBackground: Print background graphics. Defaults to false.
        :type printBackground: bool
        :param scale: Scale of the webpage rendering. Defaults to 1.
        :type scale: float
        :param paperWidth: Paper width in inches. Defaults to 8.5 inches.
        :type paperWidth: float
        :param paperHeight: Paper height in inches. Defaults to 11 inches.
        :type paperHeight: float
        :param marginTop: Top margin in inches. Defaults to 1cm (~0.4 inches).
        :type marginTop: float
        :param marginBottom: Bottom margin in inches. Defaults to 1cm (~0.4 inches).
        :type marginBottom: float
        :param marginLeft: Left margin in inches. Defaults to 1cm (~0.4 inches).
        :type marginLeft: float
        :param marginRight: Right margin in inches. Defaults to 1cm (~0.4 inches).
        :type marginRight: float
        :param pageRanges: Paper ranges to print, e.g., '1-5, 8, 11-13'. Defaults to the empty string, which means
print all pages.
        :type pageRanges: str
        :param ignoreInvalidPageRanges: Whether to silently ignore invalid but successfully parsed page ranges, such as '3-2'.
Defaults to false.
        :type ignoreInvalidPageRanges: bool
        :param headerTemplate: HTML template for the print header. Should be valid HTML markup with following
classes used to inject printing values into them:
- `date`: formatted print date
- `title`: document title
- `url`: document location
- `pageNumber`: current page number
- `totalPages`: total pages in the document

For example, `<span class=title></span>` would generate span containing the title.
        :type headerTemplate: str
        :param footerTemplate: HTML template for the print footer. Should use the same format as the `headerTemplate`.
        :type footerTemplate: str
        :param preferCSSPageSize: Whether or not to prefer page size as defined by css. Defaults to false,
in which case the content will be scaled to fit the paper size.
        :type preferCSSPageSize: bool
        """
        return (
            cls.build_send_payload("printToPDF", {
                "landscape": landscape,
                "displayHeaderFooter": displayHeaderFooter,
                "printBackground": printBackground,
                "scale": scale,
                "paperWidth": paperWidth,
                "paperHeight": paperHeight,
                "marginTop": marginTop,
                "marginBottom": marginBottom,
                "marginLeft": marginLeft,
                "marginRight": marginRight,
                "pageRanges": pageRanges,
                "ignoreInvalidPageRanges": ignoreInvalidPageRanges,
                "headerTemplate": headerTemplate,
                "footerTemplate": footerTemplate,
                "preferCSSPageSize": preferCSSPageSize,
            }),
            cls.convert_payload({
                "data": {
                    "class": str,
                    "optional": False
                },
            })
        )

    @classmethod
    def reload(cls,
               ignoreCache: Optional['bool'] = None,
               scriptToEvaluateOnLoad: Optional['str'] = None,
               ):
        """Reloads given page optionally ignoring the cache.
        :param ignoreCache: If true, browser cache is ignored (as if the user pressed Shift+refresh).
        :type ignoreCache: bool
        :param scriptToEvaluateOnLoad: If set, the script will be injected into all frames of the inspected page after reload.
Argument will be ignored if reloading dataURL origin.
        :type scriptToEvaluateOnLoad: str
        """
        return (
            cls.build_send_payload("reload", {
                "ignoreCache": ignoreCache,
                "scriptToEvaluateOnLoad": scriptToEvaluateOnLoad,
            }),
            None
        )

    @classmethod
    def removeScriptToEvaluateOnLoad(cls,
                                     identifier: Union['ScriptIdentifier'],
                                     ):
        """Deprecated, please use removeScriptToEvaluateOnNewDocument instead.
        :param identifier: 
        :type identifier: ScriptIdentifier
        """
        return (
            cls.build_send_payload("removeScriptToEvaluateOnLoad", {
                "identifier": identifier,
            }),
            None
        )

    @classmethod
    def removeScriptToEvaluateOnNewDocument(cls,
                                            identifier: Union['ScriptIdentifier'],
                                            ):
        """Removes given script from the list.
        :param identifier: 
        :type identifier: ScriptIdentifier
        """
        return (
            cls.build_send_payload("removeScriptToEvaluateOnNewDocument", {
                "identifier": identifier,
            }),
            None
        )

    @classmethod
    def screencastFrameAck(cls,
                           sessionId: Union['int'],
                           ):
        """Acknowledges that a screencast frame has been received by the frontend.
        :param sessionId: Frame number.
        :type sessionId: int
        """
        return (
            cls.build_send_payload("screencastFrameAck", {
                "sessionId": sessionId,
            }),
            None
        )

    @classmethod
    def searchInResource(cls,
                         frameId: Union['FrameId'],
                         url: Union['str'],
                         query: Union['str'],
                         caseSensitive: Optional['bool'] = None,
                         isRegex: Optional['bool'] = None,
                         ):
        """Searches for given string in resource content.
        :param frameId: Frame id for resource to search in.
        :type frameId: FrameId
        :param url: URL of the resource to search in.
        :type url: str
        :param query: String to search for.
        :type query: str
        :param caseSensitive: If true, search is case sensitive.
        :type caseSensitive: bool
        :param isRegex: If true, treats string parameter as regex.
        :type isRegex: bool
        """
        return (
            cls.build_send_payload("searchInResource", {
                "frameId": frameId,
                "url": url,
                "query": query,
                "caseSensitive": caseSensitive,
                "isRegex": isRegex,
            }),
            cls.convert_payload({
                "result": {
                    "class": [Debugger.SearchMatch],
                    "optional": False
                },
            })
        )

    @classmethod
    def setAdBlockingEnabled(cls,
                             enabled: Union['bool'],
                             ):
        """Enable Chrome's experimental ad filter on all sites.
        :param enabled: Whether to block ads.
        :type enabled: bool
        """
        return (
            cls.build_send_payload("setAdBlockingEnabled", {
                "enabled": enabled,
            }),
            None
        )

    @classmethod
    def setBypassCSP(cls,
                     enabled: Union['bool'],
                     ):
        """Enable page Content Security Policy by-passing.
        :param enabled: Whether to bypass page CSP.
        :type enabled: bool
        """
        return (
            cls.build_send_payload("setBypassCSP", {
                "enabled": enabled,
            }),
            None
        )

    @classmethod
    def setDeviceMetricsOverride(cls,
                                 width: Union['int'],
                                 height: Union['int'],
                                 deviceScaleFactor: Union['float'],
                                 mobile: Union['bool'],
                                 scale: Optional['float'] = None,
                                 screenWidth: Optional['int'] = None,
                                 screenHeight: Optional['int'] = None,
                                 positionX: Optional['int'] = None,
                                 positionY: Optional['int'] = None,
                                 dontSetVisibleSize: Optional['bool'] = None,
                                 screenOrientation: Optional['Emulation.ScreenOrientation'] = None,
                                 viewport: Optional['Viewport'] = None,
                                 ):
        """Overrides the values of device screen dimensions (window.screen.width, window.screen.height,
window.innerWidth, window.innerHeight, and "device-width"/"device-height"-related CSS media
query results).
        :param width: Overriding width value in pixels (minimum 0, maximum 10000000). 0 disables the override.
        :type width: int
        :param height: Overriding height value in pixels (minimum 0, maximum 10000000). 0 disables the override.
        :type height: int
        :param deviceScaleFactor: Overriding device scale factor value. 0 disables the override.
        :type deviceScaleFactor: float
        :param mobile: Whether to emulate mobile device. This includes viewport meta tag, overlay scrollbars, text
autosizing and more.
        :type mobile: bool
        :param scale: Scale to apply to resulting view image.
        :type scale: float
        :param screenWidth: Overriding screen width value in pixels (minimum 0, maximum 10000000).
        :type screenWidth: int
        :param screenHeight: Overriding screen height value in pixels (minimum 0, maximum 10000000).
        :type screenHeight: int
        :param positionX: Overriding view X position on screen in pixels (minimum 0, maximum 10000000).
        :type positionX: int
        :param positionY: Overriding view Y position on screen in pixels (minimum 0, maximum 10000000).
        :type positionY: int
        :param dontSetVisibleSize: Do not set visible view size, rely upon explicit setVisibleSize call.
        :type dontSetVisibleSize: bool
        :param screenOrientation: Screen orientation override.
        :type screenOrientation: Emulation.ScreenOrientation
        :param viewport: The viewport dimensions and scale. If not set, the override is cleared.
        :type viewport: Viewport
        """
        return (
            cls.build_send_payload("setDeviceMetricsOverride", {
                "width": width,
                "height": height,
                "deviceScaleFactor": deviceScaleFactor,
                "mobile": mobile,
                "scale": scale,
                "screenWidth": screenWidth,
                "screenHeight": screenHeight,
                "positionX": positionX,
                "positionY": positionY,
                "dontSetVisibleSize": dontSetVisibleSize,
                "screenOrientation": screenOrientation,
                "viewport": viewport,
            }),
            None
        )

    @classmethod
    def setDeviceOrientationOverride(cls,
                                     alpha: Union['float'],
                                     beta: Union['float'],
                                     gamma: Union['float'],
                                     ):
        """Overrides the Device Orientation.
        :param alpha: Mock alpha
        :type alpha: float
        :param beta: Mock beta
        :type beta: float
        :param gamma: Mock gamma
        :type gamma: float
        """
        return (
            cls.build_send_payload("setDeviceOrientationOverride", {
                "alpha": alpha,
                "beta": beta,
                "gamma": gamma,
            }),
            None
        )

    @classmethod
    def setFontFamilies(cls,
                        fontFamilies: Union['FontFamilies'],
                        ):
        """Set generic font families.
        :param fontFamilies: Specifies font families to set. If a font family is not specified, it won't be changed.
        :type fontFamilies: FontFamilies
        """
        return (
            cls.build_send_payload("setFontFamilies", {
                "fontFamilies": fontFamilies,
            }),
            None
        )

    @classmethod
    def setFontSizes(cls,
                     fontSizes: Union['FontSizes'],
                     ):
        """Set default font sizes.
        :param fontSizes: Specifies font sizes to set. If a font size is not specified, it won't be changed.
        :type fontSizes: FontSizes
        """
        return (
            cls.build_send_payload("setFontSizes", {
                "fontSizes": fontSizes,
            }),
            None
        )

    @classmethod
    def setDocumentContent(cls,
                           frameId: Union['FrameId'],
                           html: Union['str'],
                           ):
        """Sets given markup as the document's HTML.
        :param frameId: Frame id to set HTML for.
        :type frameId: FrameId
        :param html: HTML content to set.
        :type html: str
        """
        return (
            cls.build_send_payload("setDocumentContent", {
                "frameId": frameId,
                "html": html,
            }),
            None
        )

    @classmethod
    def setDownloadBehavior(cls,
                            behavior: Union['str'],
                            downloadPath: Optional['str'] = None,
                            ):
        """Set the behavior when downloading a file.
        :param behavior: Whether to allow all or deny all download requests, or use default Chrome behavior if
available (otherwise deny).
        :type behavior: str
        :param downloadPath: The default path to save downloaded files to. This is requred if behavior is set to 'allow'
        :type downloadPath: str
        """
        return (
            cls.build_send_payload("setDownloadBehavior", {
                "behavior": behavior,
                "downloadPath": downloadPath,
            }),
            None
        )

    @classmethod
    def setGeolocationOverride(cls,
                               latitude: Optional['float'] = None,
                               longitude: Optional['float'] = None,
                               accuracy: Optional['float'] = None,
                               ):
        """Overrides the Geolocation Position or Error. Omitting any of the parameters emulates position
unavailable.
        :param latitude: Mock latitude
        :type latitude: float
        :param longitude: Mock longitude
        :type longitude: float
        :param accuracy: Mock accuracy
        :type accuracy: float
        """
        return (
            cls.build_send_payload("setGeolocationOverride", {
                "latitude": latitude,
                "longitude": longitude,
                "accuracy": accuracy,
            }),
            None
        )

    @classmethod
    def setLifecycleEventsEnabled(cls,
                                  enabled: Union['bool'],
                                  ):
        """Controls whether page will emit lifecycle events.
        :param enabled: If true, starts emitting lifecycle events.
        :type enabled: bool
        """
        return (
            cls.build_send_payload("setLifecycleEventsEnabled", {
                "enabled": enabled,
            }),
            None
        )

    @classmethod
    def setTouchEmulationEnabled(cls,
                                 enabled: Union['bool'],
                                 configuration: Optional['str'] = None,
                                 ):
        """Toggles mouse event-based touch event emulation.
        :param enabled: Whether the touch event emulation should be enabled.
        :type enabled: bool
        :param configuration: Touch/gesture events configuration. Default: current platform.
        :type configuration: str
        """
        return (
            cls.build_send_payload("setTouchEmulationEnabled", {
                "enabled": enabled,
                "configuration": configuration,
            }),
            None
        )

    @classmethod
    def startScreencast(cls,
                        format: Optional['str'] = None,
                        quality: Optional['int'] = None,
                        maxWidth: Optional['int'] = None,
                        maxHeight: Optional['int'] = None,
                        everyNthFrame: Optional['int'] = None,
                        ):
        """Starts sending each frame using the `screencastFrame` event.
        :param format: Image compression format.
        :type format: str
        :param quality: Compression quality from range [0..100].
        :type quality: int
        :param maxWidth: Maximum screenshot width.
        :type maxWidth: int
        :param maxHeight: Maximum screenshot height.
        :type maxHeight: int
        :param everyNthFrame: Send every n-th frame.
        :type everyNthFrame: int
        """
        return (
            cls.build_send_payload("startScreencast", {
                "format": format,
                "quality": quality,
                "maxWidth": maxWidth,
                "maxHeight": maxHeight,
                "everyNthFrame": everyNthFrame,
            }),
            None
        )

    @classmethod
    def stopLoading(cls):
        """Force the page stop all navigations and pending resource fetches.
        """
        return (
            cls.build_send_payload("stopLoading", {
            }),
            None
        )

    @classmethod
    def crash(cls):
        """Crashes renderer on the IO thread, generates minidumps.
        """
        return (
            cls.build_send_payload("crash", {
            }),
            None
        )

    @classmethod
    def close(cls):
        """Tries to close page, running its beforeunload hooks, if any.
        """
        return (
            cls.build_send_payload("close", {
            }),
            None
        )

    @classmethod
    def setWebLifecycleState(cls,
                             state: Union['str'],
                             ):
        """Tries to update the web lifecycle state of the page.
It will transition the page to the given state according to:
https://github.com/WICG/web-lifecycle/
        :param state: Target lifecycle state
        :type state: str
        """
        return (
            cls.build_send_payload("setWebLifecycleState", {
                "state": state,
            }),
            None
        )

    @classmethod
    def stopScreencast(cls):
        """Stops sending each frame in the `screencastFrame`.
        """
        return (
            cls.build_send_payload("stopScreencast", {
            }),
            None
        )

    @classmethod
    def setProduceCompilationCache(cls,
                                   enabled: Union['bool'],
                                   ):
        """Forces compilation cache to be generated for every subresource script.
        :param enabled: 
        :type enabled: bool
        """
        return (
            cls.build_send_payload("setProduceCompilationCache", {
                "enabled": enabled,
            }),
            None
        )

    @classmethod
    def addCompilationCache(cls,
                            url: Union['str'],
                            data: Union['str'],
                            ):
        """Seeds compilation cache for given url. Compilation cache does not survive
cross-process navigation.
        :param url: 
        :type url: str
        :param data: Base64-encoded data
        :type data: str
        """
        return (
            cls.build_send_payload("addCompilationCache", {
                "url": url,
                "data": data,
            }),
            None
        )

    @classmethod
    def clearCompilationCache(cls):
        """Clears seeded compilation cache.
        """
        return (
            cls.build_send_payload("clearCompilationCache", {
            }),
            None
        )

    @classmethod
    def generateTestReport(cls,
                           message: Union['str'],
                           group: Optional['str'] = None,
                           ):
        """Generates a report for testing.
        :param message: Message to be displayed in the report.
        :type message: str
        :param group: Specifies the endpoint group to deliver the report to.
        :type group: str
        """
        return (
            cls.build_send_payload("generateTestReport", {
                "message": message,
                "group": group,
            }),
            None
        )

    @classmethod
    def waitForDebugger(cls):
        """Pauses page execution. Can be resumed using generic Runtime.runIfWaitingForDebugger.
        """
        return (
            cls.build_send_payload("waitForDebugger", {
            }),
            None
        )



class DomContentEventFiredEvent(BaseEvent):

    js_name = 'Page.domContentEventFired'
    hashable = []
    is_hashable = False

    def __init__(self,
                 timestamp: Union['Network.MonotonicTime', dict],
                 ):
        if isinstance(timestamp, dict):
            timestamp = Network.MonotonicTime(**timestamp)
        self.timestamp = timestamp

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class FrameAttachedEvent(BaseEvent):

    js_name = 'Page.frameAttached'
    hashable = ['parentFrameId', 'frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 parentFrameId: Union['FrameId', dict],
                 stack: Union['Runtime.StackTrace', dict, None] = None,
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId
        if isinstance(parentFrameId, dict):
            parentFrameId = FrameId(**parentFrameId)
        self.parentFrameId = parentFrameId
        if isinstance(stack, dict):
            stack = Runtime.StackTrace(**stack)
        self.stack = stack

    @classmethod
    def build_hash(cls, parentFrameId, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameClearedScheduledNavigationEvent(BaseEvent):

    js_name = 'Page.frameClearedScheduledNavigation'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameDetachedEvent(BaseEvent):

    js_name = 'Page.frameDetached'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameNavigatedEvent(BaseEvent):

    js_name = 'Page.frameNavigated'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frame: Union['Frame', dict],
                 ):
        if isinstance(frame, dict):
            frame = Frame(**frame)
        self.frame = frame

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameResizedEvent(BaseEvent):

    js_name = 'Page.frameResized'
    hashable = []
    is_hashable = False

    def __init__(self):
        pass

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class FrameRequestedNavigationEvent(BaseEvent):

    js_name = 'Page.frameRequestedNavigation'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 reason: Union['ClientNavigationReason', dict],
                 url: Union['str', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId
        if isinstance(reason, dict):
            reason = ClientNavigationReason(**reason)
        self.reason = reason
        if isinstance(url, dict):
            url = str(**url)
        self.url = url

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameScheduledNavigationEvent(BaseEvent):

    js_name = 'Page.frameScheduledNavigation'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 delay: Union['float', dict],
                 reason: Union['str', dict],
                 url: Union['str', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId
        if isinstance(delay, dict):
            delay = float(**delay)
        self.delay = delay
        if isinstance(reason, dict):
            reason = str(**reason)
        self.reason = reason
        if isinstance(url, dict):
            url = str(**url)
        self.url = url

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameStartedLoadingEvent(BaseEvent):

    js_name = 'Page.frameStartedLoading'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class FrameStoppedLoadingEvent(BaseEvent):

    js_name = 'Page.frameStoppedLoading'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class InterstitialHiddenEvent(BaseEvent):

    js_name = 'Page.interstitialHidden'
    hashable = []
    is_hashable = False

    def __init__(self):
        pass

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class InterstitialShownEvent(BaseEvent):

    js_name = 'Page.interstitialShown'
    hashable = []
    is_hashable = False

    def __init__(self):
        pass

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class JavascriptDialogClosedEvent(BaseEvent):

    js_name = 'Page.javascriptDialogClosed'
    hashable = []
    is_hashable = False

    def __init__(self,
                 result: Union['bool', dict],
                 userInput: Union['str', dict],
                 ):
        if isinstance(result, dict):
            result = bool(**result)
        self.result = result
        if isinstance(userInput, dict):
            userInput = str(**userInput)
        self.userInput = userInput

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class JavascriptDialogOpeningEvent(BaseEvent):

    js_name = 'Page.javascriptDialogOpening'
    hashable = []
    is_hashable = False

    def __init__(self,
                 url: Union['str', dict],
                 message: Union['str', dict],
                 type: Union['DialogType', dict],
                 hasBrowserHandler: Union['bool', dict],
                 defaultPrompt: Union['str', dict, None] = None,
                 ):
        if isinstance(url, dict):
            url = str(**url)
        self.url = url
        if isinstance(message, dict):
            message = str(**message)
        self.message = message
        if isinstance(type, dict):
            type = DialogType(**type)
        self.type = type
        if isinstance(hasBrowserHandler, dict):
            hasBrowserHandler = bool(**hasBrowserHandler)
        self.hasBrowserHandler = hasBrowserHandler
        if isinstance(defaultPrompt, dict):
            defaultPrompt = str(**defaultPrompt)
        self.defaultPrompt = defaultPrompt

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class LifecycleEventEvent(BaseEvent):

    js_name = 'Page.lifecycleEvent'
    hashable = ['loaderId', 'frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 loaderId: Union['Network.LoaderId', dict],
                 name: Union['str', dict],
                 timestamp: Union['Network.MonotonicTime', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId
        if isinstance(loaderId, dict):
            loaderId = Network.LoaderId(**loaderId)
        self.loaderId = loaderId
        if isinstance(name, dict):
            name = str(**name)
        self.name = name
        if isinstance(timestamp, dict):
            timestamp = Network.MonotonicTime(**timestamp)
        self.timestamp = timestamp

    @classmethod
    def build_hash(cls, loaderId, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class LoadEventFiredEvent(BaseEvent):

    js_name = 'Page.loadEventFired'
    hashable = []
    is_hashable = False

    def __init__(self,
                 timestamp: Union['Network.MonotonicTime', dict],
                 ):
        if isinstance(timestamp, dict):
            timestamp = Network.MonotonicTime(**timestamp)
        self.timestamp = timestamp

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class NavigatedWithinDocumentEvent(BaseEvent):

    js_name = 'Page.navigatedWithinDocument'
    hashable = ['frameId']
    is_hashable = True

    def __init__(self,
                 frameId: Union['FrameId', dict],
                 url: Union['str', dict],
                 ):
        if isinstance(frameId, dict):
            frameId = FrameId(**frameId)
        self.frameId = frameId
        if isinstance(url, dict):
            url = str(**url)
        self.url = url

    @classmethod
    def build_hash(cls, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class ScreencastFrameEvent(BaseEvent):

    js_name = 'Page.screencastFrame'
    hashable = ['sessionId']
    is_hashable = True

    def __init__(self,
                 data: Union['str', dict],
                 metadata: Union['ScreencastFrameMetadata', dict],
                 sessionId: Union['int', dict],
                 ):
        if isinstance(data, dict):
            data = str(**data)
        self.data = data
        if isinstance(metadata, dict):
            metadata = ScreencastFrameMetadata(**metadata)
        self.metadata = metadata
        if isinstance(sessionId, dict):
            sessionId = int(**sessionId)
        self.sessionId = sessionId

    @classmethod
    def build_hash(cls, sessionId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class ScreencastVisibilityChangedEvent(BaseEvent):

    js_name = 'Page.screencastVisibilityChanged'
    hashable = []
    is_hashable = False

    def __init__(self,
                 visible: Union['bool', dict],
                 ):
        if isinstance(visible, dict):
            visible = bool(**visible)
        self.visible = visible

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class WindowOpenEvent(BaseEvent):

    js_name = 'Page.windowOpen'
    hashable = []
    is_hashable = False

    def __init__(self,
                 url: Union['str', dict],
                 windowName: Union['str', dict],
                 windowFeatures: Union['[]', dict],
                 userGesture: Union['bool', dict],
                 ):
        if isinstance(url, dict):
            url = str(**url)
        self.url = url
        if isinstance(windowName, dict):
            windowName = str(**windowName)
        self.windowName = windowName
        if isinstance(windowFeatures, dict):
            windowFeatures = [](**windowFeatures)
        self.windowFeatures = windowFeatures
        if isinstance(userGesture, dict):
            userGesture = bool(**userGesture)
        self.userGesture = userGesture

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class CompilationCacheProducedEvent(BaseEvent):

    js_name = 'Page.compilationCacheProduced'
    hashable = []
    is_hashable = False

    def __init__(self,
                 url: Union['str', dict],
                 data: Union['str', dict],
                 ):
        if isinstance(url, dict):
            url = str(**url)
        self.url = url
        if isinstance(data, dict):
            data = str(**data)
        self.data = data

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')



# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)

# WindowID: 
WindowID = int

# WindowState: The state of the browser window.
WindowState = str

# Bounds: Browser window bounds information
class Bounds(ChromeTypeBase):
    def __init__(self,
                 left: Optional['int'] = None,
                 top: Optional['int'] = None,
                 width: Optional['int'] = None,
                 height: Optional['int'] = None,
                 windowState: Optional['WindowState'] = None,
                 ):

        self.left = left
        self.top = top
        self.width = width
        self.height = height
        self.windowState = windowState


# PermissionType: 
PermissionType = str

# Bucket: Chrome histogram bucket.
class Bucket(ChromeTypeBase):
    def __init__(self,
                 low: Union['int'],
                 high: Union['int'],
                 count: Union['int'],
                 ):

        self.low = low
        self.high = high
        self.count = count


# Histogram: Chrome histogram.
class Histogram(ChromeTypeBase):
    def __init__(self,
                 name: Union['str'],
                 sum: Union['int'],
                 count: Union['int'],
                 buckets: Union['[Bucket]'],
                 ):

        self.name = name
        self.sum = sum
        self.count = count
        self.buckets = buckets


class Browser(PayloadMixin):
    """ The Browser domain defines methods and events for browser managing.
    """
    @classmethod
    def grantPermissions(cls,
                         origin: Union['str'],
                         permissions: Union['[PermissionType]'],
                         browserContextId: Optional['Target.BrowserContextID'] = None,
                         ):
        """Grant specific permissions to the given origin and reject all others.
        :param origin: 
        :type origin: str
        :param permissions: 
        :type permissions: [PermissionType]
        :param browserContextId: BrowserContext to override permissions. When omitted, default browser context is used.
        :type browserContextId: Target.BrowserContextID
        """
        return (
            cls.build_send_payload("grantPermissions", {
                "origin": origin,
                "permissions": permissions,
                "browserContextId": browserContextId,
            }),
            None
        )

    @classmethod
    def resetPermissions(cls,
                         browserContextId: Optional['Target.BrowserContextID'] = None,
                         ):
        """Reset all permission management for all origins.
        :param browserContextId: BrowserContext to reset permissions. When omitted, default browser context is used.
        :type browserContextId: Target.BrowserContextID
        """
        return (
            cls.build_send_payload("resetPermissions", {
                "browserContextId": browserContextId,
            }),
            None
        )

    @classmethod
    def close(cls):
        """Close browser gracefully.
        """
        return (
            cls.build_send_payload("close", {
            }),
            None
        )

    @classmethod
    def crash(cls):
        """Crashes browser on the main thread.
        """
        return (
            cls.build_send_payload("crash", {
            }),
            None
        )

    @classmethod
    def crashGpuProcess(cls):
        """Crashes GPU process.
        """
        return (
            cls.build_send_payload("crashGpuProcess", {
            }),
            None
        )

    @classmethod
    def getVersion(cls):
        """Returns version information.
        """
        return (
            cls.build_send_payload("getVersion", {
            }),
            cls.convert_payload({
                "protocolVersion": {
                    "class": str,
                    "optional": False
                },
                "product": {
                    "class": str,
                    "optional": False
                },
                "revision": {
                    "class": str,
                    "optional": False
                },
                "userAgent": {
                    "class": str,
                    "optional": False
                },
                "jsVersion": {
                    "class": str,
                    "optional": False
                },
            })
        )

    @classmethod
    def getBrowserCommandLine(cls):
        """Returns the command line switches for the browser process if, and only if
--enable-automation is on the commandline.
        """
        return (
            cls.build_send_payload("getBrowserCommandLine", {
            }),
            cls.convert_payload({
                "arguments": {
                    "class": [],
                    "optional": False
                },
            })
        )

    @classmethod
    def getHistograms(cls,
                      query: Optional['str'] = None,
                      delta: Optional['bool'] = None,
                      ):
        """Get Chrome histograms.
        :param query: Requested substring in name. Only histograms which have query as a
substring in their name are extracted. An empty or absent query returns
all histograms.
        :type query: str
        :param delta: If true, retrieve delta since last call.
        :type delta: bool
        """
        return (
            cls.build_send_payload("getHistograms", {
                "query": query,
                "delta": delta,
            }),
            cls.convert_payload({
                "histograms": {
                    "class": [Histogram],
                    "optional": False
                },
            })
        )

    @classmethod
    def getHistogram(cls,
                     name: Union['str'],
                     delta: Optional['bool'] = None,
                     ):
        """Get a Chrome histogram by name.
        :param name: Requested histogram name.
        :type name: str
        :param delta: If true, retrieve delta since last call.
        :type delta: bool
        """
        return (
            cls.build_send_payload("getHistogram", {
                "name": name,
                "delta": delta,
            }),
            cls.convert_payload({
                "histogram": {
                    "class": Histogram,
                    "optional": False
                },
            })
        )

    @classmethod
    def getWindowBounds(cls,
                        windowId: Union['WindowID'],
                        ):
        """Get position and size of the browser window.
        :param windowId: Browser window id.
        :type windowId: WindowID
        """
        return (
            cls.build_send_payload("getWindowBounds", {
                "windowId": windowId,
            }),
            cls.convert_payload({
                "bounds": {
                    "class": Bounds,
                    "optional": False
                },
            })
        )

    @classmethod
    def getWindowForTarget(cls,
                           targetId: Optional['Target.TargetID'] = None,
                           ):
        """Get the browser window that contains the devtools target.
        :param targetId: Devtools agent host id. If called as a part of the session, associated targetId is used.
        :type targetId: Target.TargetID
        """
        return (
            cls.build_send_payload("getWindowForTarget", {
                "targetId": targetId,
            }),
            cls.convert_payload({
                "windowId": {
                    "class": WindowID,
                    "optional": False
                },
                "bounds": {
                    "class": Bounds,
                    "optional": False
                },
            })
        )

    @classmethod
    def setWindowBounds(cls,
                        windowId: Union['WindowID'],
                        bounds: Union['Bounds'],
                        ):
        """Set position and/or size of the browser window.
        :param windowId: Browser window id.
        :type windowId: WindowID
        :param bounds: New window bounds. The 'minimized', 'maximized' and 'fullscreen' states cannot be combined
with 'left', 'top', 'width' or 'height'. Leaves unspecified fields unchanged.
        :type bounds: Bounds
        """
        return (
            cls.build_send_payload("setWindowBounds", {
                "windowId": windowId,
                "bounds": bounds,
            }),
            None
        )

    @classmethod
    def setDockTile(cls,
                    badgeLabel: Optional['str'] = None,
                    image: Optional['str'] = None,
                    ):
        """Set dock tile details, platform-specific.
        :param badgeLabel: 
        :type badgeLabel: str
        :param image: Png encoded image.
        :type image: str
        """
        return (
            cls.build_send_payload("setDockTile", {
                "badgeLabel": badgeLabel,
                "image": image,
            }),
            None
        )




# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)
from chromewhip.protocol import target as Target

class Tethering(PayloadMixin):
    """ The Tethering domain defines methods and events for browser port binding.
    """
    @classmethod
    def bind(cls,
             port: Union['int'],
             ):
        """Request browser port binding.
        :param port: Port number to bind.
        :type port: int
        """
        return (
            cls.build_send_payload("bind", {
                "port": port,
            }),
            None
        )

    @classmethod
    def unbind(cls,
               port: Union['int'],
               ):
        """Request browser port unbinding.
        :param port: Port number to unbind.
        :type port: int
        """
        return (
            cls.build_send_payload("unbind", {
                "port": port,
            }),
            None
        )



class AcceptedEvent(BaseEvent):

    js_name = 'Tethering.accepted'
    hashable = ['connectionId']
    is_hashable = True

    def __init__(self,
                 port: Union['int', dict],
                 connectionId: Union['str', dict],
                 ):
        if isinstance(port, dict):
            port = int(**port)
        self.port = port
        if isinstance(connectionId, dict):
            connectionId = str(**connectionId)
        self.connectionId = connectionId

    @classmethod
    def build_hash(cls, connectionId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h



# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)

# RegistrationID: 
RegistrationID = str

# ServiceWorkerRegistration: ServiceWorker registration.
class ServiceWorkerRegistration(ChromeTypeBase):
    def __init__(self,
                 registrationId: Union['RegistrationID'],
                 scopeURL: Union['str'],
                 isDeleted: Union['bool'],
                 ):

        self.registrationId = registrationId
        self.scopeURL = scopeURL
        self.isDeleted = isDeleted


# ServiceWorkerVersionRunningStatus: 
ServiceWorkerVersionRunningStatus = str

# ServiceWorkerVersionStatus: 
ServiceWorkerVersionStatus = str

# ServiceWorkerVersion: ServiceWorker version.
class ServiceWorkerVersion(ChromeTypeBase):
    def __init__(self,
                 versionId: Union['str'],
                 registrationId: Union['RegistrationID'],
                 scriptURL: Union['str'],
                 runningStatus: Union['ServiceWorkerVersionRunningStatus'],
                 status: Union['ServiceWorkerVersionStatus'],
                 scriptLastModified: Optional['float'] = None,
                 scriptResponseTime: Optional['float'] = None,
                 controlledClients: Optional['[Target.TargetID]'] = None,
                 targetId: Optional['Target.TargetID'] = None,
                 ):

        self.versionId = versionId
        self.registrationId = registrationId
        self.scriptURL = scriptURL
        self.runningStatus = runningStatus
        self.status = status
        self.scriptLastModified = scriptLastModified
        self.scriptResponseTime = scriptResponseTime
        self.controlledClients = controlledClients
        self.targetId = targetId


# ServiceWorkerErrorMessage: ServiceWorker error message.
class ServiceWorkerErrorMessage(ChromeTypeBase):
    def __init__(self,
                 errorMessage: Union['str'],
                 registrationId: Union['RegistrationID'],
                 versionId: Union['str'],
                 sourceURL: Union['str'],
                 lineNumber: Union['int'],
                 columnNumber: Union['int'],
                 ):

        self.errorMessage = errorMessage
        self.registrationId = registrationId
        self.versionId = versionId
        self.sourceURL = sourceURL
        self.lineNumber = lineNumber
        self.columnNumber = columnNumber


class ServiceWorker(PayloadMixin):
    """ 
    """
    @classmethod
    def deliverPushMessage(cls,
                           origin: Union['str'],
                           registrationId: Union['RegistrationID'],
                           data: Union['str'],
                           ):
        """
        :param origin: 
        :type origin: str
        :param registrationId: 
        :type registrationId: RegistrationID
        :param data: 
        :type data: str
        """
        return (
            cls.build_send_payload("deliverPushMessage", {
                "origin": origin,
                "registrationId": registrationId,
                "data": data,
            }),
            None
        )

    @classmethod
    def disable(cls):
        """
        """
        return (
            cls.build_send_payload("disable", {
            }),
            None
        )

    @classmethod
    def dispatchSyncEvent(cls,
                          origin: Union['str'],
                          registrationId: Union['RegistrationID'],
                          tag: Union['str'],
                          lastChance: Union['bool'],
                          ):
        """
        :param origin: 
        :type origin: str
        :param registrationId: 
        :type registrationId: RegistrationID
        :param tag: 
        :type tag: str
        :param lastChance: 
        :type lastChance: bool
        """
        return (
            cls.build_send_payload("dispatchSyncEvent", {
                "origin": origin,
                "registrationId": registrationId,
                "tag": tag,
                "lastChance": lastChance,
            }),
            None
        )

    @classmethod
    def enable(cls):
        """
        """
        return (
            cls.build_send_payload("enable", {
            }),
            None
        )

    @classmethod
    def inspectWorker(cls,
                      versionId: Union['str'],
                      ):
        """
        :param versionId: 
        :type versionId: str
        """
        return (
            cls.build_send_payload("inspectWorker", {
                "versionId": versionId,
            }),
            None
        )

    @classmethod
    def setForceUpdateOnPageLoad(cls,
                                 forceUpdateOnPageLoad: Union['bool'],
                                 ):
        """
        :param forceUpdateOnPageLoad: 
        :type forceUpdateOnPageLoad: bool
        """
        return (
            cls.build_send_payload("setForceUpdateOnPageLoad", {
                "forceUpdateOnPageLoad": forceUpdateOnPageLoad,
            }),
            None
        )

    @classmethod
    def skipWaiting(cls,
                    scopeURL: Union['str'],
                    ):
        """
        :param scopeURL: 
        :type scopeURL: str
        """
        return (
            cls.build_send_payload("skipWaiting", {
                "scopeURL": scopeURL,
            }),
            None
        )

    @classmethod
    def startWorker(cls,
                    scopeURL: Union['str'],
                    ):
        """
        :param scopeURL: 
        :type scopeURL: str
        """
        return (
            cls.build_send_payload("startWorker", {
                "scopeURL": scopeURL,
            }),
            None
        )

    @classmethod
    def stopAllWorkers(cls):
        """
        """
        return (
            cls.build_send_payload("stopAllWorkers", {
            }),
            None
        )

    @classmethod
    def stopWorker(cls,
                   versionId: Union['str'],
                   ):
        """
        :param versionId: 
        :type versionId: str
        """
        return (
            cls.build_send_payload("stopWorker", {
                "versionId": versionId,
            }),
            None
        )

    @classmethod
    def unregister(cls,
                   scopeURL: Union['str'],
                   ):
        """
        :param scopeURL: 
        :type scopeURL: str
        """
        return (
            cls.build_send_payload("unregister", {
                "scopeURL": scopeURL,
            }),
            None
        )

    @classmethod
    def updateRegistration(cls,
                           scopeURL: Union['str'],
                           ):
        """
        :param scopeURL: 
        :type scopeURL: str
        """
        return (
            cls.build_send_payload("updateRegistration", {
                "scopeURL": scopeURL,
            }),
            None
        )



class WorkerErrorReportedEvent(BaseEvent):

    js_name = 'Serviceworker.workerErrorReported'
    hashable = []
    is_hashable = False

    def __init__(self,
                 errorMessage: Union['ServiceWorkerErrorMessage', dict],
                 ):
        if isinstance(errorMessage, dict):
            errorMessage = ServiceWorkerErrorMessage(**errorMessage)
        self.errorMessage = errorMessage

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class WorkerRegistrationUpdatedEvent(BaseEvent):

    js_name = 'Serviceworker.workerRegistrationUpdated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 registrations: Union['[ServiceWorkerRegistration]', dict],
                 ):
        if isinstance(registrations, dict):
            registrations = [ServiceWorkerRegistration](**registrations)
        self.registrations = registrations

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class WorkerVersionUpdatedEvent(BaseEvent):

    js_name = 'Serviceworker.workerVersionUpdated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 versions: Union['[ServiceWorkerVersion]', dict],
                 ):
        if isinstance(versions, dict):
            versions = [ServiceWorkerVersion](**versions)
        self.versions = versions

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')



# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)
from chromewhip.protocol import debugger as Debugger
from chromewhip.protocol import runtime as Runtime
from chromewhip.protocol import security as Security
from chromewhip.protocol import page as Page

# ResourceType: Resource type as it was perceived by the rendering engine.
ResourceType = str

# LoaderId: Unique loader identifier.
LoaderId = str

# RequestId: Unique request identifier.
RequestId = str

# InterceptionId: Unique intercepted request identifier.
InterceptionId = str

# ErrorReason: Network level fetch failure reason.
ErrorReason = str

# TimeSinceEpoch: UTC time in seconds, counted from January 1, 1970.
TimeSinceEpoch = float

# MonotonicTime: Monotonically increasing time in seconds since an arbitrary point in the past.
MonotonicTime = float

# Headers: Request / response headers as keys / values of JSON object.
Headers = dict

# ConnectionType: The underlying connection technology that the browser is supposedly using.
ConnectionType = str

# CookieSameSite: Represents the cookie's 'SameSite' status:https://tools.ietf.org/html/draft-west-first-party-cookies
CookieSameSite = str

# ResourceTiming: Timing information for the request.
class ResourceTiming(ChromeTypeBase):
    def __init__(self,
                 requestTime: Union['float'],
                 proxyStart: Union['float'],
                 proxyEnd: Union['float'],
                 dnsStart: Union['float'],
                 dnsEnd: Union['float'],
                 connectStart: Union['float'],
                 connectEnd: Union['float'],
                 sslStart: Union['float'],
                 sslEnd: Union['float'],
                 workerStart: Union['float'],
                 workerReady: Union['float'],
                 sendStart: Union['float'],
                 sendEnd: Union['float'],
                 pushStart: Union['float'],
                 pushEnd: Union['float'],
                 receiveHeadersEnd: Union['float'],
                 ):

        self.requestTime = requestTime
        self.proxyStart = proxyStart
        self.proxyEnd = proxyEnd
        self.dnsStart = dnsStart
        self.dnsEnd = dnsEnd
        self.connectStart = connectStart
        self.connectEnd = connectEnd
        self.sslStart = sslStart
        self.sslEnd = sslEnd
        self.workerStart = workerStart
        self.workerReady = workerReady
        self.sendStart = sendStart
        self.sendEnd = sendEnd
        self.pushStart = pushStart
        self.pushEnd = pushEnd
        self.receiveHeadersEnd = receiveHeadersEnd


# ResourcePriority: Loading priority of a resource request.
ResourcePriority = str

# Request: HTTP request data.
class Request(ChromeTypeBase):
    def __init__(self,
                 url: Union['str'],
                 method: Union['str'],
                 headers: Union['Headers'],
                 initialPriority: Union['ResourcePriority'],
                 referrerPolicy: Union['str'],
                 urlFragment: Optional['str'] = None,
                 postData: Optional['str'] = None,
                 hasPostData: Optional['bool'] = None,
                 mixedContentType: Optional['Security.MixedContentType'] = None,
                 isLinkPreload: Optional['bool'] = None,
                 ):

        self.url = url
        self.urlFragment = urlFragment
        self.method = method
        self.headers = headers
        self.postData = postData
        self.hasPostData = hasPostData
        self.mixedContentType = mixedContentType
        self.initialPriority = initialPriority
        self.referrerPolicy = referrerPolicy
        self.isLinkPreload = isLinkPreload


# SignedCertificateTimestamp: Details of a signed certificate timestamp (SCT).
class SignedCertificateTimestamp(ChromeTypeBase):
    def __init__(self,
                 status: Union['str'],
                 origin: Union['str'],
                 logDescription: Union['str'],
                 logId: Union['str'],
                 timestamp: Union['TimeSinceEpoch'],
                 hashAlgorithm: Union['str'],
                 signatureAlgorithm: Union['str'],
                 signatureData: Union['str'],
                 ):

        self.status = status
        self.origin = origin
        self.logDescription = logDescription
        self.logId = logId
        self.timestamp = timestamp
        self.hashAlgorithm = hashAlgorithm
        self.signatureAlgorithm = signatureAlgorithm
        self.signatureData = signatureData


# SecurityDetails: Security details about a request.
class SecurityDetails(ChromeTypeBase):
    def __init__(self,
                 protocol: Union['str'],
                 keyExchange: Union['str'],
                 cipher: Union['str'],
                 certificateId: Union['Security.CertificateId'],
                 subjectName: Union['str'],
                 sanList: Union['[]'],
                 issuer: Union['str'],
                 validFrom: Union['TimeSinceEpoch'],
                 validTo: Union['TimeSinceEpoch'],
                 signedCertificateTimestampList: Union['[SignedCertificateTimestamp]'],
                 certificateTransparencyCompliance: Union['CertificateTransparencyCompliance'],
                 keyExchangeGroup: Optional['str'] = None,
                 mac: Optional['str'] = None,
                 ):

        self.protocol = protocol
        self.keyExchange = keyExchange
        self.keyExchangeGroup = keyExchangeGroup
        self.cipher = cipher
        self.mac = mac
        self.certificateId = certificateId
        self.subjectName = subjectName
        self.sanList = sanList
        self.issuer = issuer
        self.validFrom = validFrom
        self.validTo = validTo
        self.signedCertificateTimestampList = signedCertificateTimestampList
        self.certificateTransparencyCompliance = certificateTransparencyCompliance


# CertificateTransparencyCompliance: Whether the request complied with Certificate Transparency policy.
CertificateTransparencyCompliance = str

# BlockedReason: The reason why request was blocked.
BlockedReason = str

# Response: HTTP response data.
class Response(ChromeTypeBase):
    def __init__(self,
                 url: Union['str'],
                 status: Union['int'],
                 statusText: Union['str'],
                 headers: Union['Headers'],
                 mimeType: Union['str'],
                 connectionReused: Union['bool'],
                 connectionId: Union['float'],
                 encodedDataLength: Union['float'],
                 securityState: Union['Security.SecurityState'],
                 headersText: Optional['str'] = None,
                 requestHeaders: Optional['Headers'] = None,
                 requestHeadersText: Optional['str'] = None,
                 remoteIPAddress: Optional['str'] = None,
                 remotePort: Optional['int'] = None,
                 fromDiskCache: Optional['bool'] = None,
                 fromServiceWorker: Optional['bool'] = None,
                 timing: Optional['ResourceTiming'] = None,
                 protocol: Optional['str'] = None,
                 securityDetails: Optional['SecurityDetails'] = None,
                 ):

        self.url = url
        self.status = status
        self.statusText = statusText
        self.headers = headers
        self.headersText = headersText
        self.mimeType = mimeType
        self.requestHeaders = requestHeaders
        self.requestHeadersText = requestHeadersText
        self.connectionReused = connectionReused
        self.connectionId = connectionId
        self.remoteIPAddress = remoteIPAddress
        self.remotePort = remotePort
        self.fromDiskCache = fromDiskCache
        self.fromServiceWorker = fromServiceWorker
        self.encodedDataLength = encodedDataLength
        self.timing = timing
        self.protocol = protocol
        self.securityState = securityState
        self.securityDetails = securityDetails


# WebSocketRequest: WebSocket request data.
class WebSocketRequest(ChromeTypeBase):
    def __init__(self,
                 headers: Union['Headers'],
                 ):

        self.headers = headers


# WebSocketResponse: WebSocket response data.
class WebSocketResponse(ChromeTypeBase):
    def __init__(self,
                 status: Union['int'],
                 statusText: Union['str'],
                 headers: Union['Headers'],
                 headersText: Optional['str'] = None,
                 requestHeaders: Optional['Headers'] = None,
                 requestHeadersText: Optional['str'] = None,
                 ):

        self.status = status
        self.statusText = statusText
        self.headers = headers
        self.headersText = headersText
        self.requestHeaders = requestHeaders
        self.requestHeadersText = requestHeadersText


# WebSocketFrame: WebSocket message data. This represents an entire WebSocket message, not just a fragmented frame as the name suggests.
class WebSocketFrame(ChromeTypeBase):
    def __init__(self,
                 opcode: Union['float'],
                 mask: Union['bool'],
                 payloadData: Union['str'],
                 ):

        self.opcode = opcode
        self.mask = mask
        self.payloadData = payloadData


# CachedResource: Information about the cached resource.
class CachedResource(ChromeTypeBase):
    def __init__(self,
                 url: Union['str'],
                 type: Union['ResourceType'],
                 bodySize: Union['float'],
                 response: Optional['Response'] = None,
                 ):

        self.url = url
        self.type = type
        self.response = response
        self.bodySize = bodySize


# Initiator: Information about the request initiator.
class Initiator(ChromeTypeBase):
    def __init__(self,
                 type: Union['str'],
                 stack: Optional['Runtime.StackTrace'] = None,
                 url: Optional['str'] = None,
                 lineNumber: Optional['float'] = None,
                 ):

        self.type = type
        self.stack = stack
        self.url = url
        self.lineNumber = lineNumber


# Cookie: Cookie object
class Cookie(ChromeTypeBase):
    def __init__(self,
                 name: Union['str'],
                 value: Union['str'],
                 domain: Union['str'],
                 path: Union['str'],
                 expires: Union['float'],
                 size: Union['int'],
                 httpOnly: Union['bool'],
                 secure: Union['bool'],
                 session: Union['bool'],
                 sameSite: Optional['CookieSameSite'] = None,
                 ):

        self.name = name
        self.value = value
        self.domain = domain
        self.path = path
        self.expires = expires
        self.size = size
        self.httpOnly = httpOnly
        self.secure = secure
        self.session = session
        self.sameSite = sameSite


# CookieParam: Cookie parameter object
class CookieParam(ChromeTypeBase):
    def __init__(self,
                 name: Union['str'],
                 value: Union['str'],
                 url: Optional['str'] = None,
                 domain: Optional['str'] = None,
                 path: Optional['str'] = None,
                 secure: Optional['bool'] = None,
                 httpOnly: Optional['bool'] = None,
                 sameSite: Optional['CookieSameSite'] = None,
                 expires: Optional['TimeSinceEpoch'] = None,
                 ):

        self.name = name
        self.value = value
        self.url = url
        self.domain = domain
        self.path = path
        self.secure = secure
        self.httpOnly = httpOnly
        self.sameSite = sameSite
        self.expires = expires


# AuthChallenge: Authorization challenge for HTTP status code 401 or 407.
class AuthChallenge(ChromeTypeBase):
    def __init__(self,
                 origin: Union['str'],
                 scheme: Union['str'],
                 realm: Union['str'],
                 source: Optional['str'] = None,
                 ):

        self.source = source
        self.origin = origin
        self.scheme = scheme
        self.realm = realm


# AuthChallengeResponse: Response to an AuthChallenge.
class AuthChallengeResponse(ChromeTypeBase):
    def __init__(self,
                 response: Union['str'],
                 username: Optional['str'] = None,
                 password: Optional['str'] = None,
                 ):

        self.response = response
        self.username = username
        self.password = password


# InterceptionStage: Stages of the interception to begin intercepting. Request will intercept before the request issent. Response will intercept after the response is received.
InterceptionStage = str

# RequestPattern: Request pattern for interception.
class RequestPattern(ChromeTypeBase):
    def __init__(self,
                 urlPattern: Optional['str'] = None,
                 resourceType: Optional['ResourceType'] = None,
                 interceptionStage: Optional['InterceptionStage'] = None,
                 ):

        self.urlPattern = urlPattern
        self.resourceType = resourceType
        self.interceptionStage = interceptionStage


# SignedExchangeSignature: Information about a signed exchange signature.https://wicg.github.io/webpackage/draft-yasskin-httpbis-origin-signed-exchanges-impl.html#rfc.section.3.1
class SignedExchangeSignature(ChromeTypeBase):
    def __init__(self,
                 label: Union['str'],
                 signature: Union['str'],
                 integrity: Union['str'],
                 validityUrl: Union['str'],
                 date: Union['int'],
                 expires: Union['int'],
                 certUrl: Optional['str'] = None,
                 certSha256: Optional['str'] = None,
                 certificates: Optional['[]'] = None,
                 ):

        self.label = label
        self.signature = signature
        self.integrity = integrity
        self.certUrl = certUrl
        self.certSha256 = certSha256
        self.validityUrl = validityUrl
        self.date = date
        self.expires = expires
        self.certificates = certificates


# SignedExchangeHeader: Information about a signed exchange header.https://wicg.github.io/webpackage/draft-yasskin-httpbis-origin-signed-exchanges-impl.html#cbor-representation
class SignedExchangeHeader(ChromeTypeBase):
    def __init__(self,
                 requestUrl: Union['str'],
                 responseCode: Union['int'],
                 responseHeaders: Union['Headers'],
                 signatures: Union['[SignedExchangeSignature]'],
                 ):

        self.requestUrl = requestUrl
        self.responseCode = responseCode
        self.responseHeaders = responseHeaders
        self.signatures = signatures


# SignedExchangeErrorField: Field type for a signed exchange related error.
SignedExchangeErrorField = str

# SignedExchangeError: Information about a signed exchange response.
class SignedExchangeError(ChromeTypeBase):
    def __init__(self,
                 message: Union['str'],
                 signatureIndex: Optional['int'] = None,
                 errorField: Optional['SignedExchangeErrorField'] = None,
                 ):

        self.message = message
        self.signatureIndex = signatureIndex
        self.errorField = errorField


# SignedExchangeInfo: Information about a signed exchange response.
class SignedExchangeInfo(ChromeTypeBase):
    def __init__(self,
                 outerResponse: Union['Response'],
                 header: Optional['SignedExchangeHeader'] = None,
                 securityDetails: Optional['SecurityDetails'] = None,
                 errors: Optional['[SignedExchangeError]'] = None,
                 ):

        self.outerResponse = outerResponse
        self.header = header
        self.securityDetails = securityDetails
        self.errors = errors


class Network(PayloadMixin):
    """ Network domain allows tracking network activities of the page. It exposes information about http,
file, data and other requests and responses, their headers, bodies, timing, etc.
    """
    @classmethod
    def canClearBrowserCache(cls):
        """Tells whether clearing browser cache is supported.
        """
        return (
            cls.build_send_payload("canClearBrowserCache", {
            }),
            cls.convert_payload({
                "result": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def canClearBrowserCookies(cls):
        """Tells whether clearing browser cookies is supported.
        """
        return (
            cls.build_send_payload("canClearBrowserCookies", {
            }),
            cls.convert_payload({
                "result": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def canEmulateNetworkConditions(cls):
        """Tells whether emulation of network conditions is supported.
        """
        return (
            cls.build_send_payload("canEmulateNetworkConditions", {
            }),
            cls.convert_payload({
                "result": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def clearBrowserCache(cls):
        """Clears browser cache.
        """
        return (
            cls.build_send_payload("clearBrowserCache", {
            }),
            None
        )

    @classmethod
    def clearBrowserCookies(cls):
        """Clears browser cookies.
        """
        return (
            cls.build_send_payload("clearBrowserCookies", {
            }),
            None
        )

    @classmethod
    def continueInterceptedRequest(cls,
                                   interceptionId: Union['InterceptionId'],
                                   errorReason: Optional['ErrorReason'] = None,
                                   rawResponse: Optional['str'] = None,
                                   url: Optional['str'] = None,
                                   method: Optional['str'] = None,
                                   postData: Optional['str'] = None,
                                   headers: Optional['Headers'] = None,
                                   authChallengeResponse: Optional['AuthChallengeResponse'] = None,
                                   ):
        """Response to Network.requestIntercepted which either modifies the request to continue with any
modifications, or blocks it, or completes it with the provided response bytes. If a network
fetch occurs as a result which encounters a redirect an additional Network.requestIntercepted
event will be sent with the same InterceptionId.
        :param interceptionId: 
        :type interceptionId: InterceptionId
        :param errorReason: If set this causes the request to fail with the given reason. Passing `Aborted` for requests
marked with `isNavigationRequest` also cancels the navigation. Must not be set in response
to an authChallenge.
        :type errorReason: ErrorReason
        :param rawResponse: If set the requests completes using with the provided base64 encoded raw response, including
HTTP status line and headers etc... Must not be set in response to an authChallenge.
        :type rawResponse: str
        :param url: If set the request url will be modified in a way that's not observable by page. Must not be
set in response to an authChallenge.
        :type url: str
        :param method: If set this allows the request method to be overridden. Must not be set in response to an
authChallenge.
        :type method: str
        :param postData: If set this allows postData to be set. Must not be set in response to an authChallenge.
        :type postData: str
        :param headers: If set this allows the request headers to be changed. Must not be set in response to an
authChallenge.
        :type headers: Headers
        :param authChallengeResponse: Response to a requestIntercepted with an authChallenge. Must not be set otherwise.
        :type authChallengeResponse: AuthChallengeResponse
        """
        return (
            cls.build_send_payload("continueInterceptedRequest", {
                "interceptionId": interceptionId,
                "errorReason": errorReason,
                "rawResponse": rawResponse,
                "url": url,
                "method": method,
                "postData": postData,
                "headers": headers,
                "authChallengeResponse": authChallengeResponse,
            }),
            None
        )

    @classmethod
    def deleteCookies(cls,
                      name: Union['str'],
                      url: Optional['str'] = None,
                      domain: Optional['str'] = None,
                      path: Optional['str'] = None,
                      ):
        """Deletes browser cookies with matching name and url or domain/path pair.
        :param name: Name of the cookies to remove.
        :type name: str
        :param url: If specified, deletes all the cookies with the given name where domain and path match
provided URL.
        :type url: str
        :param domain: If specified, deletes only cookies with the exact domain.
        :type domain: str
        :param path: If specified, deletes only cookies with the exact path.
        :type path: str
        """
        return (
            cls.build_send_payload("deleteCookies", {
                "name": name,
                "url": url,
                "domain": domain,
                "path": path,
            }),
            None
        )

    @classmethod
    def disable(cls):
        """Disables network tracking, prevents network events from being sent to the client.
        """
        return (
            cls.build_send_payload("disable", {
            }),
            None
        )

    @classmethod
    def emulateNetworkConditions(cls,
                                 offline: Union['bool'],
                                 latency: Union['float'],
                                 downloadThroughput: Union['float'],
                                 uploadThroughput: Union['float'],
                                 connectionType: Optional['ConnectionType'] = None,
                                 ):
        """Activates emulation of network conditions.
        :param offline: True to emulate internet disconnection.
        :type offline: bool
        :param latency: Minimum latency from request sent to response headers received (ms).
        :type latency: float
        :param downloadThroughput: Maximal aggregated download throughput (bytes/sec). -1 disables download throttling.
        :type downloadThroughput: float
        :param uploadThroughput: Maximal aggregated upload throughput (bytes/sec).  -1 disables upload throttling.
        :type uploadThroughput: float
        :param connectionType: Connection type if known.
        :type connectionType: ConnectionType
        """
        return (
            cls.build_send_payload("emulateNetworkConditions", {
                "offline": offline,
                "latency": latency,
                "downloadThroughput": downloadThroughput,
                "uploadThroughput": uploadThroughput,
                "connectionType": connectionType,
            }),
            None
        )

    @classmethod
    def enable(cls,
               maxTotalBufferSize: Optional['int'] = None,
               maxResourceBufferSize: Optional['int'] = None,
               maxPostDataSize: Optional['int'] = None,
               ):
        """Enables network tracking, network events will now be delivered to the client.
        :param maxTotalBufferSize: Buffer size in bytes to use when preserving network payloads (XHRs, etc).
        :type maxTotalBufferSize: int
        :param maxResourceBufferSize: Per-resource buffer size in bytes to use when preserving network payloads (XHRs, etc).
        :type maxResourceBufferSize: int
        :param maxPostDataSize: Longest post body size (in bytes) that would be included in requestWillBeSent notification
        :type maxPostDataSize: int
        """
        return (
            cls.build_send_payload("enable", {
                "maxTotalBufferSize": maxTotalBufferSize,
                "maxResourceBufferSize": maxResourceBufferSize,
                "maxPostDataSize": maxPostDataSize,
            }),
            None
        )

    @classmethod
    def getAllCookies(cls):
        """Returns all browser cookies. Depending on the backend support, will return detailed cookie
information in the `cookies` field.
        """
        return (
            cls.build_send_payload("getAllCookies", {
            }),
            cls.convert_payload({
                "cookies": {
                    "class": [Cookie],
                    "optional": False
                },
            })
        )

    @classmethod
    def getCertificate(cls,
                       origin: Union['str'],
                       ):
        """Returns the DER-encoded certificate.
        :param origin: Origin to get certificate for.
        :type origin: str
        """
        return (
            cls.build_send_payload("getCertificate", {
                "origin": origin,
            }),
            cls.convert_payload({
                "tableNames": {
                    "class": [],
                    "optional": False
                },
            })
        )

    @classmethod
    def getCookies(cls,
                   urls: Optional['[]'] = None,
                   ):
        """Returns all browser cookies for the current URL. Depending on the backend support, will return
detailed cookie information in the `cookies` field.
        :param urls: The list of URLs for which applicable cookies will be fetched
        :type urls: []
        """
        return (
            cls.build_send_payload("getCookies", {
                "urls": urls,
            }),
            cls.convert_payload({
                "cookies": {
                    "class": [Cookie],
                    "optional": False
                },
            })
        )

    @classmethod
    def getResponseBody(cls,
                        requestId: Union['RequestId'],
                        ):
        """Returns content served for the given request.
        :param requestId: Identifier of the network request to get content for.
        :type requestId: RequestId
        """
        return (
            cls.build_send_payload("getResponseBody", {
                "requestId": requestId,
            }),
            cls.convert_payload({
                "body": {
                    "class": str,
                    "optional": False
                },
                "base64Encoded": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def getRequestPostData(cls,
                           requestId: Union['RequestId'],
                           ):
        """Returns post data sent with the request. Returns an error when no data was sent with the request.
        :param requestId: Identifier of the network request to get content for.
        :type requestId: RequestId
        """
        return (
            cls.build_send_payload("getRequestPostData", {
                "requestId": requestId,
            }),
            cls.convert_payload({
                "postData": {
                    "class": str,
                    "optional": False
                },
            })
        )

    @classmethod
    def getResponseBodyForInterception(cls,
                                       interceptionId: Union['InterceptionId'],
                                       ):
        """Returns content served for the given currently intercepted request.
        :param interceptionId: Identifier for the intercepted request to get body for.
        :type interceptionId: InterceptionId
        """
        return (
            cls.build_send_payload("getResponseBodyForInterception", {
                "interceptionId": interceptionId,
            }),
            cls.convert_payload({
                "body": {
                    "class": str,
                    "optional": False
                },
                "base64Encoded": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def takeResponseBodyForInterceptionAsStream(cls,
                                                interceptionId: Union['InterceptionId'],
                                                ):
        """Returns a handle to the stream representing the response body. Note that after this command,
the intercepted request can't be continued as is -- you either need to cancel it or to provide
the response body. The stream only supports sequential read, IO.read will fail if the position
is specified.
        :param interceptionId: 
        :type interceptionId: InterceptionId
        """
        return (
            cls.build_send_payload("takeResponseBodyForInterceptionAsStream", {
                "interceptionId": interceptionId,
            }),
            cls.convert_payload({
                "stream": {
                    "class": IO.StreamHandle,
                    "optional": False
                },
            })
        )

    @classmethod
    def replayXHR(cls,
                  requestId: Union['RequestId'],
                  ):
        """This method sends a new XMLHttpRequest which is identical to the original one. The following
parameters should be identical: method, url, async, request body, extra headers, withCredentials
attribute, user, password.
        :param requestId: Identifier of XHR to replay.
        :type requestId: RequestId
        """
        return (
            cls.build_send_payload("replayXHR", {
                "requestId": requestId,
            }),
            None
        )

    @classmethod
    def searchInResponseBody(cls,
                             requestId: Union['RequestId'],
                             query: Union['str'],
                             caseSensitive: Optional['bool'] = None,
                             isRegex: Optional['bool'] = None,
                             ):
        """Searches for given string in response content.
        :param requestId: Identifier of the network response to search.
        :type requestId: RequestId
        :param query: String to search for.
        :type query: str
        :param caseSensitive: If true, search is case sensitive.
        :type caseSensitive: bool
        :param isRegex: If true, treats string parameter as regex.
        :type isRegex: bool
        """
        return (
            cls.build_send_payload("searchInResponseBody", {
                "requestId": requestId,
                "query": query,
                "caseSensitive": caseSensitive,
                "isRegex": isRegex,
            }),
            cls.convert_payload({
                "result": {
                    "class": [Debugger.SearchMatch],
                    "optional": False
                },
            })
        )

    @classmethod
    def setBlockedURLs(cls,
                       urls: Union['[]'],
                       ):
        """Blocks URLs from loading.
        :param urls: URL patterns to block. Wildcards ('*') are allowed.
        :type urls: []
        """
        return (
            cls.build_send_payload("setBlockedURLs", {
                "urls": urls,
            }),
            None
        )

    @classmethod
    def setBypassServiceWorker(cls,
                               bypass: Union['bool'],
                               ):
        """Toggles ignoring of service worker for each request.
        :param bypass: Bypass service worker and load from network.
        :type bypass: bool
        """
        return (
            cls.build_send_payload("setBypassServiceWorker", {
                "bypass": bypass,
            }),
            None
        )

    @classmethod
    def setCacheDisabled(cls,
                         cacheDisabled: Union['bool'],
                         ):
        """Toggles ignoring cache for each request. If `true`, cache will not be used.
        :param cacheDisabled: Cache disabled state.
        :type cacheDisabled: bool
        """
        return (
            cls.build_send_payload("setCacheDisabled", {
                "cacheDisabled": cacheDisabled,
            }),
            None
        )

    @classmethod
    def setCookie(cls,
                  name: Union['str'],
                  value: Union['str'],
                  url: Optional['str'] = None,
                  domain: Optional['str'] = None,
                  path: Optional['str'] = None,
                  secure: Optional['bool'] = None,
                  httpOnly: Optional['bool'] = None,
                  sameSite: Optional['CookieSameSite'] = None,
                  expires: Optional['TimeSinceEpoch'] = None,
                  ):
        """Sets a cookie with the given cookie data; may overwrite equivalent cookies if they exist.
        :param name: Cookie name.
        :type name: str
        :param value: Cookie value.
        :type value: str
        :param url: The request-URI to associate with the setting of the cookie. This value can affect the
default domain and path values of the created cookie.
        :type url: str
        :param domain: Cookie domain.
        :type domain: str
        :param path: Cookie path.
        :type path: str
        :param secure: True if cookie is secure.
        :type secure: bool
        :param httpOnly: True if cookie is http-only.
        :type httpOnly: bool
        :param sameSite: Cookie SameSite type.
        :type sameSite: CookieSameSite
        :param expires: Cookie expiration date, session cookie if not set
        :type expires: TimeSinceEpoch
        """
        return (
            cls.build_send_payload("setCookie", {
                "name": name,
                "value": value,
                "url": url,
                "domain": domain,
                "path": path,
                "secure": secure,
                "httpOnly": httpOnly,
                "sameSite": sameSite,
                "expires": expires,
            }),
            cls.convert_payload({
                "success": {
                    "class": bool,
                    "optional": False
                },
            })
        )

    @classmethod
    def setCookies(cls,
                   cookies: Union['[CookieParam]'],
                   ):
        """Sets given cookies.
        :param cookies: Cookies to be set.
        :type cookies: [CookieParam]
        """
        return (
            cls.build_send_payload("setCookies", {
                "cookies": cookies,
            }),
            None
        )

    @classmethod
    def setDataSizeLimitsForTest(cls,
                                 maxTotalSize: Union['int'],
                                 maxResourceSize: Union['int'],
                                 ):
        """For testing.
        :param maxTotalSize: Maximum total buffer size.
        :type maxTotalSize: int
        :param maxResourceSize: Maximum per-resource size.
        :type maxResourceSize: int
        """
        return (
            cls.build_send_payload("setDataSizeLimitsForTest", {
                "maxTotalSize": maxTotalSize,
                "maxResourceSize": maxResourceSize,
            }),
            None
        )

    @classmethod
    def setExtraHTTPHeaders(cls,
                            headers: Union['Headers'],
                            ):
        """Specifies whether to always send extra HTTP headers with the requests from this page.
        :param headers: Map with extra HTTP headers.
        :type headers: Headers
        """
        return (
            cls.build_send_payload("setExtraHTTPHeaders", {
                "headers": headers,
            }),
            None
        )

    @classmethod
    def setRequestInterception(cls,
                               patterns: Union['[RequestPattern]'],
                               ):
        """Sets the requests to intercept that match the provided patterns and optionally resource types.
        :param patterns: Requests matching any of these patterns will be forwarded and wait for the corresponding
continueInterceptedRequest call.
        :type patterns: [RequestPattern]
        """
        return (
            cls.build_send_payload("setRequestInterception", {
                "patterns": patterns,
            }),
            None
        )

    @classmethod
    def setUserAgentOverride(cls,
                             userAgent: Union['str'],
                             acceptLanguage: Optional['str'] = None,
                             platform: Optional['str'] = None,
                             ):
        """Allows overriding user agent with the given string.
        :param userAgent: User agent to use.
        :type userAgent: str
        :param acceptLanguage: Browser langugage to emulate.
        :type acceptLanguage: str
        :param platform: The platform navigator.platform should return.
        :type platform: str
        """
        return (
            cls.build_send_payload("setUserAgentOverride", {
                "userAgent": userAgent,
                "acceptLanguage": acceptLanguage,
                "platform": platform,
            }),
            None
        )



class DataReceivedEvent(BaseEvent):

    js_name = 'Network.dataReceived'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 dataLength: Union['int', dict],
                 encodedDataLength: Union['int', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(dataLength, dict):
            dataLength = int(**dataLength)
        self.dataLength = dataLength
        if isinstance(encodedDataLength, dict):
            encodedDataLength = int(**encodedDataLength)
        self.encodedDataLength = encodedDataLength

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class EventSourceMessageReceivedEvent(BaseEvent):

    js_name = 'Network.eventSourceMessageReceived'
    hashable = ['eventId', 'requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 eventName: Union['str', dict],
                 eventId: Union['str', dict],
                 data: Union['str', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(eventName, dict):
            eventName = str(**eventName)
        self.eventName = eventName
        if isinstance(eventId, dict):
            eventId = str(**eventId)
        self.eventId = eventId
        if isinstance(data, dict):
            data = str(**data)
        self.data = data

    @classmethod
    def build_hash(cls, eventId, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class LoadingFailedEvent(BaseEvent):

    js_name = 'Network.loadingFailed'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 type: Union['ResourceType', dict],
                 errorText: Union['str', dict],
                 canceled: Union['bool', dict, None] = None,
                 blockedReason: Union['BlockedReason', dict, None] = None,
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(type, dict):
            type = ResourceType(**type)
        self.type = type
        if isinstance(errorText, dict):
            errorText = str(**errorText)
        self.errorText = errorText
        if isinstance(canceled, dict):
            canceled = bool(**canceled)
        self.canceled = canceled
        if isinstance(blockedReason, dict):
            blockedReason = BlockedReason(**blockedReason)
        self.blockedReason = blockedReason

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class LoadingFinishedEvent(BaseEvent):

    js_name = 'Network.loadingFinished'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 encodedDataLength: Union['float', dict],
                 shouldReportCorbBlocking: Union['bool', dict, None] = None,
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(encodedDataLength, dict):
            encodedDataLength = float(**encodedDataLength)
        self.encodedDataLength = encodedDataLength
        if isinstance(shouldReportCorbBlocking, dict):
            shouldReportCorbBlocking = bool(**shouldReportCorbBlocking)
        self.shouldReportCorbBlocking = shouldReportCorbBlocking

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class RequestInterceptedEvent(BaseEvent):

    js_name = 'Network.requestIntercepted'
    hashable = ['interceptionId', 'requestId', 'frameId']
    is_hashable = True

    def __init__(self,
                 interceptionId: Union['InterceptionId', dict],
                 request: Union['Request', dict],
                 frameId: Union['Page.FrameId', dict],
                 resourceType: Union['ResourceType', dict],
                 isNavigationRequest: Union['bool', dict],
                 isDownload: Union['bool', dict, None] = None,
                 redirectUrl: Union['str', dict, None] = None,
                 authChallenge: Union['AuthChallenge', dict, None] = None,
                 responseErrorReason: Union['ErrorReason', dict, None] = None,
                 responseStatusCode: Union['int', dict, None] = None,
                 responseHeaders: Union['Headers', dict, None] = None,
                 requestId: Union['RequestId', dict, None] = None,
                 ):
        if isinstance(interceptionId, dict):
            interceptionId = InterceptionId(**interceptionId)
        self.interceptionId = interceptionId
        if isinstance(request, dict):
            request = Request(**request)
        self.request = request
        if isinstance(frameId, dict):
            frameId = Page.FrameId(**frameId)
        self.frameId = frameId
        if isinstance(resourceType, dict):
            resourceType = ResourceType(**resourceType)
        self.resourceType = resourceType
        if isinstance(isNavigationRequest, dict):
            isNavigationRequest = bool(**isNavigationRequest)
        self.isNavigationRequest = isNavigationRequest
        if isinstance(isDownload, dict):
            isDownload = bool(**isDownload)
        self.isDownload = isDownload
        if isinstance(redirectUrl, dict):
            redirectUrl = str(**redirectUrl)
        self.redirectUrl = redirectUrl
        if isinstance(authChallenge, dict):
            authChallenge = AuthChallenge(**authChallenge)
        self.authChallenge = authChallenge
        if isinstance(responseErrorReason, dict):
            responseErrorReason = ErrorReason(**responseErrorReason)
        self.responseErrorReason = responseErrorReason
        if isinstance(responseStatusCode, dict):
            responseStatusCode = int(**responseStatusCode)
        self.responseStatusCode = responseStatusCode
        if isinstance(responseHeaders, dict):
            responseHeaders = Headers(**responseHeaders)
        self.responseHeaders = responseHeaders
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId

    @classmethod
    def build_hash(cls, interceptionId, requestId, frameId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class RequestServedFromCacheEvent(BaseEvent):

    js_name = 'Network.requestServedFromCache'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class RequestWillBeSentEvent(BaseEvent):

    js_name = 'Network.requestWillBeSent'
    hashable = ['frameId', 'requestId', 'loaderId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 loaderId: Union['LoaderId', dict],
                 documentURL: Union['str', dict],
                 request: Union['Request', dict],
                 timestamp: Union['MonotonicTime', dict],
                 wallTime: Union['TimeSinceEpoch', dict],
                 initiator: Union['Initiator', dict],
                 redirectResponse: Union['Response', dict, None] = None,
                 type: Union['ResourceType', dict, None] = None,
                 frameId: Union['Page.FrameId', dict, None] = None,
                 hasUserGesture: Union['bool', dict, None] = None,
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(loaderId, dict):
            loaderId = LoaderId(**loaderId)
        self.loaderId = loaderId
        if isinstance(documentURL, dict):
            documentURL = str(**documentURL)
        self.documentURL = documentURL
        if isinstance(request, dict):
            request = Request(**request)
        self.request = request
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(wallTime, dict):
            wallTime = TimeSinceEpoch(**wallTime)
        self.wallTime = wallTime
        if isinstance(initiator, dict):
            initiator = Initiator(**initiator)
        self.initiator = initiator
        if isinstance(redirectResponse, dict):
            redirectResponse = Response(**redirectResponse)
        self.redirectResponse = redirectResponse
        if isinstance(type, dict):
            type = ResourceType(**type)
        self.type = type
        if isinstance(frameId, dict):
            frameId = Page.FrameId(**frameId)
        self.frameId = frameId
        if isinstance(hasUserGesture, dict):
            hasUserGesture = bool(**hasUserGesture)
        self.hasUserGesture = hasUserGesture

    @classmethod
    def build_hash(cls, frameId, requestId, loaderId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class ResourceChangedPriorityEvent(BaseEvent):

    js_name = 'Network.resourceChangedPriority'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 newPriority: Union['ResourcePriority', dict],
                 timestamp: Union['MonotonicTime', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(newPriority, dict):
            newPriority = ResourcePriority(**newPriority)
        self.newPriority = newPriority
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class SignedExchangeReceivedEvent(BaseEvent):

    js_name = 'Network.signedExchangeReceived'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 info: Union['SignedExchangeInfo', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(info, dict):
            info = SignedExchangeInfo(**info)
        self.info = info

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class ResponseReceivedEvent(BaseEvent):

    js_name = 'Network.responseReceived'
    hashable = ['frameId', 'requestId', 'loaderId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 loaderId: Union['LoaderId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 type: Union['ResourceType', dict],
                 response: Union['Response', dict],
                 frameId: Union['Page.FrameId', dict, None] = None,
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(loaderId, dict):
            loaderId = LoaderId(**loaderId)
        self.loaderId = loaderId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(type, dict):
            type = ResourceType(**type)
        self.type = type
        if isinstance(response, dict):
            response = Response(**response)
        self.response = response
        if isinstance(frameId, dict):
            frameId = Page.FrameId(**frameId)
        self.frameId = frameId

    @classmethod
    def build_hash(cls, frameId, requestId, loaderId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketClosedEvent(BaseEvent):

    js_name = 'Network.webSocketClosed'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketCreatedEvent(BaseEvent):

    js_name = 'Network.webSocketCreated'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 url: Union['str', dict],
                 initiator: Union['Initiator', dict, None] = None,
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(url, dict):
            url = str(**url)
        self.url = url
        if isinstance(initiator, dict):
            initiator = Initiator(**initiator)
        self.initiator = initiator

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketFrameErrorEvent(BaseEvent):

    js_name = 'Network.webSocketFrameError'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 errorMessage: Union['str', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(errorMessage, dict):
            errorMessage = str(**errorMessage)
        self.errorMessage = errorMessage

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketFrameReceivedEvent(BaseEvent):

    js_name = 'Network.webSocketFrameReceived'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 response: Union['WebSocketFrame', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(response, dict):
            response = WebSocketFrame(**response)
        self.response = response

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketFrameSentEvent(BaseEvent):

    js_name = 'Network.webSocketFrameSent'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 response: Union['WebSocketFrame', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(response, dict):
            response = WebSocketFrame(**response)
        self.response = response

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketHandshakeResponseReceivedEvent(BaseEvent):

    js_name = 'Network.webSocketHandshakeResponseReceived'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 response: Union['WebSocketResponse', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(response, dict):
            response = WebSocketResponse(**response)
        self.response = response

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class WebSocketWillSendHandshakeRequestEvent(BaseEvent):

    js_name = 'Network.webSocketWillSendHandshakeRequest'
    hashable = ['requestId']
    is_hashable = True

    def __init__(self,
                 requestId: Union['RequestId', dict],
                 timestamp: Union['MonotonicTime', dict],
                 wallTime: Union['TimeSinceEpoch', dict],
                 request: Union['WebSocketRequest', dict],
                 ):
        if isinstance(requestId, dict):
            requestId = RequestId(**requestId)
        self.requestId = requestId
        if isinstance(timestamp, dict):
            timestamp = MonotonicTime(**timestamp)
        self.timestamp = timestamp
        if isinstance(wallTime, dict):
            wallTime = TimeSinceEpoch(**wallTime)
        self.wallTime = wallTime
        if isinstance(request, dict):
            request = WebSocketRequest(**request)
        self.request = request

    @classmethod
    def build_hash(cls, requestId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h
